# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-03 17:02
from __future__ import unicode_literals

from django.db import migrations
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('python_management', '0004_removed_virtual_env_field_from_global_requests'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pythonmanagementsynchronizerequest',
            name='libraries_to_install',
            field=waldur_core.core.fields.JSONField(blank=True, default=list, help_text='List of libraries to install'),
        ),
        migrations.AlterField(
            model_name='pythonmanagementsynchronizerequest',
            name='libraries_to_remove',
            field=waldur_core.core.fields.JSONField(blank=True, default=list, help_text='List of libraries to remove'),
        ),
    ]
