# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-04-18 14:11
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0047_privateservicesettings_sharedservicesettings'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='balancehistory',
            name='customer',
        ),
        migrations.RemoveField(
            model_name='customer',
            name='balance',
        ),
        migrations.DeleteModel(
            name='BalanceHistory',
        ),
    ]
