{% extends "admin_tools/dashboard/module.html" %}
{% load i18n %}
{% block module_content %}
<ul class="{{ module.layout }}">
    {% spaceless %}
    {% for child in module.children %}
    <li class="{% cycle 'odd' 'even' %}">
    <a{{ child.attrs|safe }}>{% trans child.title|capfirst %}</a>
    <p style="word-break: break-all;">{{ child.error }}</p>
    </li>
    {% endfor %}
    {% endspaceless %}
</ul>
{% endblock %}
