# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-06-14 15:37
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('invoices', '0021_remove_pending_offering_item'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='paymentdetails',
            name='customer',
        ),
        migrations.DeleteModel(
            name='PaymentDetails',
        ),
    ]
