{% extends 'experts/base.html' %}

{% block title %}
A new request for experts has been published by {{ request.customer.name }}
{% endblock title %}

{% block header %}
{{ request.customer.name }} is looking for experts to help with: {{ request.name }}.
{% endblock header %}

{% block body %}
{% if request.planned_budget > 0 %}
<tr>
    <td class="content-block">
        Estimated budget is <strong>{{ request.planned_budget }} {{ currency_name }}</strong>.
    </td>
</tr>
{% endif %}
<tr>
    <td class="content-block">
        Visit <a href="{{ request_link }}">{{ site_name }}</a> to find out more details and submit your proposal.
    </td>
</tr>
{% endblock body %}
