# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-10-25 09:56
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0052_customer_subnets'),
        ('experts', '0009_description_text_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='expertrequest',
            name='customer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, 
                                    to='structure.Customer'),
        ),
        migrations.AddField(
            model_name='expertrequest',
            name='project_name',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='expertrequest',
            name='project_uuid',
            field=models.CharField(blank=True, max_length=32),
        ),
        migrations.AlterField(
            model_name='expertrequest',
            name='project',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+',
                                    to='structure.Project'),
        ),
        migrations.AddField(
            model_name='expertcontract',
            name='team_customer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='structure.Customer'),
        ),
        migrations.AddField(
            model_name='expertcontract',
            name='team_name',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='expertcontract',
            name='team_uuid',
            field=models.CharField(blank=True, max_length=32),
        ),
        migrations.AlterField(
            model_name='expertcontract',
            name='team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='structure.Project'),
        ),
    ]
