# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-08-06 12:56
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('openstack_tenant', '0035_unique_floating_ip'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='internalip',
            unique_together=set([]),
        ),
    ]
