# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-20 12:17
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0014_remove_offering_preferred_language'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='item',
            new_name='orderitem'
        )
    ]
