# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-03-23 15:33
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_jira', '0014_refactor_issue_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='attachment',
            name='thumbnail',
            field=models.FileField(blank=True, null=True, upload_to='jira_attachments_thumbnails'),
        ),
    ]
