# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-12-29 12:41
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0052_customer_subnets'),
        ('waldur_jira', '0007_add_priority'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='issuetype',
            unique_together=set([('settings', 'backend_id')]),
        ),
        migrations.AlterUniqueTogether(
            name='priority',
            unique_together=set([('settings', 'backend_id')]),
        ),
    ]
