class PythonManagementConstants(object):
    INSTALL_PYTHON_ENVIRONMENT = 'install_python_environment'
    SYNCHRONIZE_PACKAGES = 'synchronize_packages'
    FIND_INSTALLED_LIBRARIES_FOR_VIRTUAL_ENVIRONMENT = 'find_installed_libraries_for_virtual_environment'
    FIND_INSTALLED_VIRTUAL_ENVIRONMENTS = 'find_installed_virtual_environments'
    DELETE_VIRTUAL_ENVIRONMENT = 'delete_virtual_environment'
    DELETE_PYTHON_ENVIRONMENT = 'delete_python_environment'
