{% extends 'experts/base.html' %}

{% block title %}
The contract for expert request under {{ customer_name }} / {{ project_name }} has been created
{% endblock title %}

{% block header %}
The contract for expert request under {{ customer_name }} / {{ project_name }} has been created.
{% endblock header %}

{% block body %}
<tr>
    <td class="content-block">
        Please go to <a href="{{ request_link }}">{{ site_name }}</a> to see it.
    </td>
</tr>
{% endblock body %}
