# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-03-02 08:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_jira', '0012_backend_id_null'),
    ]

    operations = [
        migrations.AddField(
            model_name='issue',
            name='assignee',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='issue',
            name='creator_name',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name='issue',
            name='resolution_date',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
