{% extends 'experts/base.html' %}

{% block title %}
A new bid has been added by {{ bid.team.customer.name }} to your request for {{ request.name }}
{% endblock title %}

{% block header %}
A new bid has been added by {{ bid.team.customer.name }} to your request for {{ request.name }}.
{% endblock header %}

{% block body %}
<tr>
    <td class="content-block">
        Go to <a href="{{ request_link }}">{{ site_name }}</a> to see the details.
    </td>
</tr>
{% endblock body %}
