# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-03-21 09:26
from __future__ import unicode_literals

from django.db import migrations
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0014_unique_backend_id_for_attachment'),
    ]

    operations = [
        migrations.AddField(
            model_name='offering',
            name='report',
            field=waldur_core.core.fields.JSONField(blank=True),
        ),
    ]
