# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-04-12 09:05
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('openstack_tenant', '0030_add_volume_image_name'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='internalip',
            unique_together=set([('instance', 'subnet')]),
        ),
    ]
