# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-03-07 19:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0043_servicesettings_geolocations'),
    ]

    operations = [
        migrations.AlterField(
            model_name='servicesettings',
            name='terms_of_services',
            field=models.URLField(blank=True, max_length=255),
        ),
    ]
