# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-10-18 08:54
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('experts', '0008_expertrequest_extra'),
    ]

    operations = [
        migrations.AlterField(
            model_name='expertrequest',
            name='description',
            field=models.TextField(blank=True),
        ),
    ]
