# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-02-08 10:06
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0013_add_more_backend_fields'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attachment',
            name='backend_id',
            field=models.CharField(max_length=255, unique=True),
        ),
    ]
