Client  {{ client.name }} <br />
registration in accordance with laws of  {{ client.laws }} <br />
with number  {{ client.number }} <br />
represented by its {{ client.representative }}  <br />
<br />
Expert {{ expert.name }}  <br />
registration in accordance with laws of {{ expert.laws }}  <br />
with number  {{ expert.number }} <br />
represented by its  {{ expert.representative }} <br />

<p>1.1 The Client using Company's Platform has requested and the Expert using Company's Platform
    has offered to execute following services: {{ service.description }}</p>

<p>2.1 The Client pays for the Services performed by the Expert agreed remuneration in amount of:
{{ remuneration.value }} {{ remuneration.currency }} hereinafter referred to as the Remuneration.
The Remuneration is {{ remuneration.type }}.</p>

<p>2.2.5 after the execution of the Services the Expert changes the status of the
Services to {{ status }};
Platform automatically informs the Client about this fact;</p>

<p>2.3.1 the Service and payment period is stated in the Service description and it is {{ period }},
    hereinafter referred to as the Payment period.</p>
