# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-06-29 12:52
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0008_native'),
    ]

    operations = [
        migrations.AlterField(
            model_name='offering',
            name='rating',
            field=models.IntegerField(help_text='Rating is value from 1 to 5.', null=True, validators=[django.core.validators.MaxValueValidator(5), django.core.validators.MinValueValidator(1)]),
        ),
    ]
